Suppress anti-aliasing in -monoMode
[xboard.git] / draw.c
diff --git a/draw.c b/draw.c
index d317507..193f65c 100644 (file)
--- a/draw.c
+++ b/draw.c
@@ -78,10 +78,6 @@ extern char *getenv();
 // [HGM] bitmaps: put before incuding the bitmaps / pixmaps, to know how many piece types there are.
 #include "common.h"
 
-#if HAVE_LIBXPM
-#include "pixmaps/pixmaps.h"
-#endif
-
 #include "frontend.h"
 #include "backend.h"
 #include "xevalgraph.h"
@@ -115,6 +111,7 @@ cairo_surface_t *csBoardWindow;
 static cairo_surface_t *pngPieceImages[2][(int)BlackPawn+4];   // png 256 x 256 images
 static cairo_surface_t *pngPieceBitmaps[2][(int)BlackPawn];    // scaled pieces as used
 static cairo_surface_t *pngPieceBitmaps2[2][(int)BlackPawn+4]; // scaled pieces in store
+static RsvgHandle *svgPieces[2][(int)BlackPawn+4]; // vector pieces in store
 static cairo_surface_t *pngBoardBitmap[2];
 int useTexture, textureW[2], textureH[2];
 
@@ -223,44 +220,11 @@ char *pngPieceNames[] = // must be in same order as internal piece encoding
   "GoldKnight", "GoldLance", "GoldPawn", "GoldSilver", NULL
 };
 
-cairo_surface_t *
-ConvertPixmap (int color, int piece)
-{
-  int i, j, stride, f, colcode[10], w, b;
-  char ch[10];
-  cairo_surface_t *res;
-  XpmPieces *p = builtInXpms + 10;
-  char **pixels = p->xpm[piece % BlackPawn][2*color];
-  int *buf;
-  sscanf(pixels[0], "%*d %*d %d", &f);
-  sscanf(appData.whitePieceColor+1, "%x", &w);
-  sscanf(appData.blackPieceColor+1, "%x", &b);
-  res = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, p->size, p->size);
-  stride = cairo_image_surface_get_stride(res);
-  buf = (int *) cairo_image_surface_get_data(res);
-  for(i=0; i<f; i++) {
-    ch[i] = pixels[i+1][0];
-    colcode[i] = 0;
-    if(strstr(pixels[i+1], "black")) colcode[i] = 0xFF000000; // 0xFF000000 + (color ? b : 0);
-    if(strstr(pixels[i+1], "white")) colcode[i] = 0xFFFFFFCC; // 0xFF000000 + w;
-  }
-  for(i=0; i<p->size; i++) {
-    for(j=0; j<p->size; j++) {
-      char c = pixels[i+f+1][j];
-      int k;
-      for(k=0; ch[k] != c && k < f; k++);
-      buf[i*p->size + j] = colcode[k];
-    }
-  }
-  cairo_surface_mark_dirty(res);
-  return res;
-}
-
-int
-LoadSVG(char *dir, int color, int piece)
+RsvgHandle *
+LoadSVG (char *dir, int color, int piece)
 {
     char buf[MSG_SIZ];
-  RsvgHandle *svg=NULL;
+  RsvgHandle *svg=svgPieces[color][piece];
   RsvgDimensionData svg_dimensions;
   GError **svgerror=NULL;
   cairo_surface_t *img;
@@ -268,7 +232,7 @@ LoadSVG(char *dir, int color, int piece)
 
     snprintf(buf, MSG_SIZ, "%s/%s%s.svg", dir, color ? "Black" : "White", pngPieceNames[piece]);
 
-    if(svg = rsvg_handle_new_from_file(buf,svgerror)) {
+    if(svg || *dir && (svg = rsvg_handle_new_from_file(buf,svgerror))) {
 
       rsvg_handle_get_dimensions(svg, &svg_dimensions);
       img = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, squareSize,  squareSize);
@@ -281,11 +245,10 @@ LoadSVG(char *dir, int color, int piece)
         pngPieceImages[color][piece] = img;
       }
       cairo_destroy(cr);
-      rsvg_handle_close (svg,NULL);
 
-      return 1;
+      return svg;
     }
-    return 0;
+    return NULL;
 }
 
 static void
@@ -298,19 +261,26 @@ ScaleOnePiece (int color, int piece)
 
   g_type_init ();
 
-  if(*appData.svgDirectory) { // try to freshly render svg pieces first, always from file, to supply the source bitmap
-    LoadSVG(appData.svgDirectory, color, piece);
+  if(!svgPieces[color][piece]) { // try to freshly render cached svg pieces first, to supply the source bitmap
+    svgPieces[color][piece] = LoadSVG("", color, piece); // this fills pngPieceImages if we had cached svg with bitmap of wanted size
   }
 
-  if((img = pngPieceImages[color][piece]) == NULL) { // if PNG file for this piece was not yet read, read it now and store it
-    if(!*appData.pieceDirectory) img = ConvertPixmap(color, piece); else {
+  if(!pngPieceImages[color][piece]) { // we don't have cached bitmap (implying we did not have cached svg)
+    if(*appData.pieceDirectory) { // user specified piece directory
       snprintf(buf, MSG_SIZ, "%s/%s%s.png", appData.pieceDirectory, color ? "Black" : "White", pngPieceNames[piece]);
-      img = cairo_image_surface_create_from_png (buf);
-      if(cairo_surface_status(img) != CAIRO_STATUS_SUCCESS) img = ConvertPixmap(color, piece);
+      pngPieceImages[color][piece] = img = cairo_image_surface_create_from_png (buf); // try if there are png pieces there
+      if(cairo_surface_status(img) != CAIRO_STATUS_SUCCESS) { // there were not
+       svgPieces[color][piece] = LoadSVG(appData.pieceDirectory, color, piece); // so try if he has svg there
+      }
     }
   }
-  pngPieceImages[color][piece] = img;
 
+  if(!pngPieceImages[color][piece]) { // we still did not manage to acquire a piece bitmap
+    if(!(svgPieces[color][piece] = LoadSVG(appData.svgDirectory, color, piece))) // try to fall back on installed svg
+      DisplayError(_("No default pieces installed\nSelect your own -pieceImageDirectory"), 0); // give up
+  }
+
+  img = pngPieceImages[color][piece];
 
   // create new bitmap to hold scaled piece image (and remove any old)
   if(pngPieceBitmaps2[color][piece]) cairo_surface_destroy (pngPieceBitmaps2[color][piece]);
@@ -341,6 +311,11 @@ ScaleOnePiece (int color, int piece)
        buf[i*stride + j] = c & 0xFF000000; // alpha channel is kept at same opacity
        buf[i*stride + j] += ((int)(f*(p&0xFF0000)) & 0xFF0000) + ((int)(f*(p&0xFF00)) & 0xFF00) + (int)(f*(p&0xFF)); // add desired fraction of new color
        if(color) buf[i*stride + j] += r | r << 8 | r << 16; // details on black pieces get their weight added in pure white
+       if(appData.monoMode) {
+           if(a < 64) buf[i*stride + j] = 0; // if not opaque enough, totally transparent
+           else if(2*r < a) buf[i*stride + j] = 0xFF000000; // if not light enough, totally black
+            else buf[i*stride + j] = 0xFFFFFFFF; // otherwise white
+       }
     }
     cairo_surface_mark_dirty(cs);
   }
@@ -374,6 +349,8 @@ InitDrawingParams (int reloadPieces)
     for(i=0; i<2; i++) for(p=0; p<BlackPawn+4; p++) {
        if(pngPieceImages[i][p]) cairo_surface_destroy(pngPieceImages[i][p]);
        pngPieceImages[i][p] = NULL;
+       if(svgPieces[i][p]) rsvg_handle_close(svgPieces[i][p], NULL);
+       svgPieces[i][p] = NULL;
     }
     CreateAnyPieces();
 }